﻿#include "toptemplateclass1.h"
#include <QtWidgets>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <twidget/tsplitter.h>
#include <twidget/ttableview.h>
#include <twidget/tsearchentry.h>
#include <twidget/tpagetool.h>

/*!
 * \class TopTemplateClass1
 * \inmodule templateclass
 * \brief 该模板只有一个表格，支持搜索分页
 */

TopTemplateClass1::TopTemplateClass1(const QString &iModuleName,
                                     const QVariantMap &iUrlPars,
                                     QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
    initUi();
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mTableView);
    QTimer::singleShot(0, this, SLOT(refresh()));
}

TopTemplateClass1::~TopTemplateClass1()
{
    saveSizeState();
    saveObjectState(mTableView);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 刷新数据，\a iResetPage 表示是否要重置分页
 */
void TopTemplateClass1::refresh(bool iResetPage)
{
    QString sql = config("table.data_set.db_sql").toString();
    if (sql.isEmpty()) {
        TSqlSelectorV2 selector;
        selector.setTable(mTableConf.dbTableName);
        selector.setField(mTableConf.queryFields);
        selector.setReturnRowCount(true);
        // 搜索分页
        TopClassHelper::handleSearchPageOnQuery(mSearchEntry, mPageTool, iResetPage, &selector);
        // 排序
        TopClassHelper::handleOrderOnQuery(mTableView, &mTableConf, &selector);
        loading(ttr("Loading"));
        QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
        unloading();
        TDataResponse response(data.toMap());
        if (response.hasError()) {
            alertError(ttr("Load data failed!"));
        } else {
            if (mPageTool != nullptr) {
                mPageTool->setRowCount(response.dataCount(), true);
            }
            QVariantList tableData = response.data().toList();
            TopClassHelper::formatTableData(this, &mTableConf, tableData);
            mTableView->loadData(tableData);
            alertOk(ttr("Data loaded"));
        }
    } else {
        // @todo
    }
}

/*!
 * \brief 表格控件对象
 */
TTableView *TopTemplateClass1::tableView() const
{
    return mTableView;
}

/*!
 * \brief 当前选中条目的primaryKey
 */
QVariantList TopTemplateClass1::selectedItems() const
{
    return mSelectedLst;
}

/*!
 * \brief 当前选中条目的所有数据
 */
QVariantList TopTemplateClass1::selectedDataMaps() const
{
    return mTableView->selectedRowDataMaps(true);
}

/*!
 * \brief 当选中发生改变时的槽函数
 */
void TopTemplateClass1::onSelectionChanged()
{
    mSelectedLst = mTableView->selectedPrimaryKeys();
    refreshActionState();
}

/*!
 * \brief 当分页发生改变时的槽函数
 */
void TopTemplateClass1::onPageChanged()
{
    refresh(false);
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass1::initUi()
{
    TSplitter *bodySplitter = new TSplitter(this);
    setCentralWidget(bodySplitter);

    mBodyWgt = new QWidget(this);
    QVBoxLayout *bodyLayout = new QVBoxLayout(mBodyWgt);
    bodyLayout->setMargin(0);
    bodyLayout->setSpacing(0);

    mTableView = new TTableView(this);
    mTableView->setObjectName("mTableView");
    connect(mTableView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refresh()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));
    mTableView->setDataKeyList(mTableConf.dataKeys);
    mTableView->setPrimaryKey(mTableConf.primaryKey);
    QVariantList headerItems = mTableConf.horizontalHeaders;
    headerItems.prepend(QVariant());
    mTableView->setHeaderItem(headerItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
        if (mSearchEntry != nullptr) {
            connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refresh()));
            mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mTableView->headerItem()));
            mSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("TABLEVIEW_POPUP"))) {
        mTableView->setContextMenu(tablePopup);
    }

    bodyLayout->addWidget(mTableView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mPageTool = qobject_cast<TPageTool *>(uim()->getWidget("BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mPageTool != nullptr) {
            mPageTool->setPageSizeVisible(true);
            connect(mPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onPageChanged()));
        }
    }

    bodySplitter->addWidget(mBodyWgt);
    bodySplitter->setStretchFactor(0, 1);
}
