﻿#include "toptemplateclass1a.h"
#include <QtWidgets>
#include <tbaseutil/tconcurrent.h>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <twidget/tsplitter.h>
#include <twidget/ttreeview.h>
#include <twidget/tsearchentry.h>

/*!
 * \class TopTemplateClass1A
 * \inmodule templateclass
 * \brief 该模板只有一个TreeView，支持搜索
 */

TopTemplateClass1A::TopTemplateClass1A(const QString &iModuleName,
                                       const QVariantMap &iUrlPars,
                                       QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "view", &mTreeConf);
    initUi();
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mTreeView);
    QTimer::singleShot(0, this, SLOT(refresh()));
}

TopTemplateClass1A::~TopTemplateClass1A()
{
    saveSizeState();
    saveObjectState(mTreeView);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 刷新数据
 */
void TopTemplateClass1A::refresh()
{
    if (mTreeConf.dbSql.isEmpty() && mTreeConf.dbTableName.isEmpty()) {
        return;
    }

    QString groupKey = config("view.group_key").toString();
    QString parentKey = config("view.parent_key").toString();
    QString childKey = config("view.child_key").toString();
    QString seqKey = config("view.seq_key").toString();

    if (mTreeConf.dbSql.isEmpty()) {
        TSqlSelectorV2 selector;
        selector.setTable(mTreeConf.dbTableName);
        QStringList fields = mTreeConf.queryFields;
        fields.removeOne("_category");
        selector.setField(fields);
        selector.setReturnRowCount(false);
        loading(ttr("Loading"));
        QVariant data = doThreadWork(new TopClassSqlThread(this),
                                     TOPSQLTHREAD_SELECT_ARRAYMAP,
                                     QVariant::fromValue(selector));
        unloading();
        TDataResponse response(data.toMap());
        if (response.hasError()) {
            alertError(ttr("Load data failed!"));
        } else {
            QVariantList tableData = response.data().toList();
            if (!groupKey.isEmpty()) {
                loadDataByGrouping(tableData, groupKey, seqKey);
            } else if (!parentKey.isEmpty() && !childKey.isEmpty()) {
                loadDataByParent(tableData, parentKey, childKey, seqKey);
            } else {
                Q_ASSERT(false);
            }
            alertOk(ttr("Data loaded"));
        }
    } else {
        TError err;
        loading(ttr("Loading"));
        QVariantList tableData = TConcurrent::runSync([this, &err](){
            TSqlQueryV2 query(T_SQLCNT_POOL->getSqlDatabase());
            QVariantList v = query.selectArrayMap(mTreeConf.dbSql, QVariantMap());
            if (query.lastError().isValid()) {
                err = query.lastError();
                return QVariantList();
            }
            return v;
        }).toList();
        unloading();
        if (err.isValid()) {
            alertError(ttr("Load data failed!"), err.text());
            return;
        }
        loadTableData(tableData, groupKey, parentKey, seqKey);
        alertOk(ttr("Data loaded"));
    }
}

/*!
 * \brief 树形控件对象
 */
TTreeView *TopTemplateClass1A::treeView() const
{
    return mTreeView;
}

/*!
 * \brief 根据配置生成的查询数据库的字段
 */
QStringList TopTemplateClass1A::queryFields() const
{
    return mTreeConf.queryFields;
}

/*!
 * \brief 当前选中条目的primaryKey
 */
QVariantList TopTemplateClass1A::selectedItems() const
{
    return mSelectedLst;
}

/*!
 * \brief 当前选中条目的所有数据
 */
QVariantList TopTemplateClass1A::selectedDataMaps() const
{
    return mTreeView->selectedRowDataMaps(true);
}

/*!
 * \brief 导入树形结构的数据
 * [
 *   {
 *     "name": "",
 *     "CHILDREN": [
 *       {
 *         "name": ""
 *       }
 *     ]
 *   }
 * ]
 */
void TopTemplateClass1A::loadTreeData(const QVariantList &iTreeData,
                                      const QString &iChildKey)
{
    mTreeView->loadTreeData(iTreeData, iChildKey);
}

/*!
 * \brief 导入列表数据，此接口已废弃
 * \param iGroupKey 分组Key
 * \param iParentKey 父结点Key, 此值为空时只按iGroupKey分组，树形只有一级
 * \param iSeqKey 同级结点排序的Key
 */
void TopTemplateClass1A::loadTableData(const QVariantList &iTableData,
                                       const QString &iGroupKey,
                                       const QString &iParentKey,
                                       const QString &iSeqKey)
{
    Q_UNUSED(iSeqKey)

    QVariantList tableData = iTableData;
    TopClassHelper::formatTableData(this, &mTreeConf, tableData);
    QVariantList data;
    if (iParentKey.isEmpty()) {
        QVariantMap categoryMap;
        for (const auto & i : tableData) {
            QVariantMap iMap = i.toMap();
            QString category = iMap.value(iGroupKey).toString();
            QVariantList items = categoryMap.value(category).toList();
            items.append(i);
            categoryMap.insert(category, items);
        }
        QMapIterator<QString, QVariant> iter(categoryMap);
        while (iter.hasNext()) {
            iter.next();
            QVariantMap item;
            item.insert("_category", iter.key());
            item.insert("CHILDREN", iter.value());
            data.append(item);
        }
    } else {
        // 此接口已废弃，不支持此分支
        Q_ASSERT(false);
    }
    mTreeView->loadTreeData(data);
}

/*!
 * \brief 导入列表数据，根据 \a iGroupKey  进行分组，此树形结构只有一级
 */
void TopTemplateClass1A::loadDataByGrouping(const QVariantList &iTableData,
                                            const QString &iGroupKey,
                                            const QString &iSeqKey)
{
    Q_UNUSED(iSeqKey)
    QVariantList tableData = iTableData;
    TopClassHelper::formatTableData(this, &mTreeConf, tableData);
    QVariantList data;
    QVariantMap categoryMap;
    for (const auto & i : tableData) {
        QVariantMap iMap = i.toMap();
        QString category = iMap.value(iGroupKey).toString();
        QVariantList items = categoryMap.value(category).toList();
        items.append(i);
        categoryMap.insert(category, items);
    }
    QMapIterator<QString, QVariant> iter(categoryMap);
    while (iter.hasNext()) {
        iter.next();
        QVariantMap item;
        item.insert("_category", iter.key());
        item.insert("CHILDREN", iter.value());
        data.append(item);
    }
    mTreeView->loadTreeData(data);
}

/*!
 * \brief 导入列表数据，根据 \a iParentKey 和 \a iChildKey 生成树，此树形结构可能有多级
 */
void TopTemplateClass1A::loadDataByParent(const QVariantList &iTableData,
                                          const QString &iParentKey,
                                          const QString &iChildKey,
                                          const QString &iSeqKey)
{
    QVariantList tableData = iTableData;
    TopClassHelper::formatTableData(this, &mTreeConf, tableData);
    mTreeView->loadTreeData(buildTree(tableData, iParentKey, iChildKey, iSeqKey));
}

/*!
 * \brief 当选中项发生改变时的槽函数
 */
void TopTemplateClass1A::onSelectionChanged()
{
    mSelectedLst = mTreeView->selectedPrimaryKeys();
    refreshActionState();
}

/*!
 * \brief 搜索
 */
void TopTemplateClass1A::onSearch(const QString &iText, const QVariant &iFields)
{
    QRegExp regex(iText);
    regex.setPatternSyntax(QRegExp::Wildcard);
    regex.setCaseSensitivity(Qt::CaseInsensitive);
    auto searchCols = iFields.toStringList();
    if (searchCols.isEmpty()) {
        searchCols = mSearchCols;
    }
    mTreeView->findRows(regex, searchCols);
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass1A::initUi()
{
    TSplitter *bodySplitter = new TSplitter(this);
    setCentralWidget(bodySplitter);

    mBodyWgt = new QWidget(this);
    QVBoxLayout *bodyLayout = new QVBoxLayout(mBodyWgt);
    bodyLayout->setMargin(0);
    bodyLayout->setSpacing(0);

    mTreeView = new TTreeView(this);
    mTreeView->setObjectName("mTreeView");
    mTreeView->header()->setStretchLastSection(true);
    connect(mTreeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));
    QStringList dataKeys = mTreeConf.dataKeys;
    dataKeys.append("_category");
    mTreeView->setDataKeyList(dataKeys);
    mTreeView->setPrimaryKey(mTreeConf.primaryKey);
    mTreeView->setHeaderItem(mTreeConf.horizontalHeaders);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
        if (mSearchEntry != nullptr) {
            auto searchLst = TDataParse::headerItem2searchList(mTreeView->headerItem());
            for (auto i : searchLst) {
                mSearchCols.push_back(i.toMap().value("name").toString());
            }
            mSearchEntry->setOptionList(searchLst);
            mSearchEntry->setPlaceholderText(ttr("Search %1"));
            connect(mSearchEntry, SIGNAL(search(QString,QVariant)),
                    this, SLOT(onSearch(QString,QVariant)));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("TREEVIEW_POPUP"))) {
        mTreeView->setContextMenu(tablePopup);
    }

    bodyLayout->addWidget(mTreeView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
    }

    bodySplitter->addWidget(mBodyWgt);
    bodySplitter->setStretchFactor(0, 1);
}

/*!
 * \brief 根据列表数据生成树形数据
 */
QVariantList TopTemplateClass1A::buildTree(const QVariantList &iDataLst,
                                           const QString &iParentKey,
                                           const QString &iChildKey,
                                           const QString &iSeqKey)
{
    if (iDataLst.isEmpty()) {
        return QVariantList();
    }

    QVariantList ret;
    QVariantList toplevel;
    QVariantMap parentDataMap;
    QVariantMap tmpDataMap;

    auto sequencePred = [iSeqKey](const QVariant &left, const QVariant &right) ->bool {
        return left.toMap().value(iSeqKey).toInt() < right.toMap().value(iSeqKey).toInt();
    };
    std::function<QVariantList(const QVariantMap &)> getChildren;
    getChildren = [&getChildren, &parentDataMap, &iChildKey, &sequencePred](const QVariantMap &iRowMap) ->QVariantList  {
        if (parentDataMap.contains(iRowMap.value(iChildKey).toString())) {
            QVariantList childLst;
            QVariantList tmp = parentDataMap.value(iRowMap.value(iChildKey).toString()).toList();
            for (const auto & child : tmp) {
                QVariantMap childMap = child.toMap();
                QVariantList children  = getChildren(childMap);
                if (!children.isEmpty()) {
                    std::sort(children.begin(), children.end(), sequencePred);
                    childMap.insert("CHILDREN", children);
                }
                childLst.append(childMap);
            }
            return childLst;
        } else {
            return QVariantList();
        }
    };

    for (const auto & i : iDataLst) {
        QVariantMap iMap = i.toMap();
        QVariantList childrenList = parentDataMap.value(
                    iMap.value(iParentKey).toString()).toList();
        childrenList.append(iMap);
        parentDataMap.insert(iMap.value(iParentKey).toString(), childrenList);
        tmpDataMap.insert(iMap.value(iChildKey).toString(), iMap);
    }

    for (const auto & i : iDataLst) {
        QVariantMap dataMap = i.toMap();
        QString parentId = dataMap.value(iParentKey).toString();
        if (!tmpDataMap.contains(parentId)) {
            toplevel.append(dataMap);
        }
    }

    for (const auto & i : toplevel) {
        QVariantMap iMap = i.toMap();
        QVariantList children = getChildren(iMap);
        if (!children.isEmpty()) {
            std::sort(children.begin(), children.end(), sequencePred);
            iMap.insert("CHILDREN", children);
        }
        ret.append(iMap);
    }
    std::sort(ret.begin(), ret.end(), sequencePred);
    return ret;
}
