﻿#include "toptemplateclass2.h"
#include <QtWidgets>
#include <tbaseutil/tconcurrent.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasshelper.h>
#include <topcore/topclasssqlthread.h>
#include <twidget/tuiloader.h>

/*!
 * \class TopTemplateClass2
 * \inmodule templateclass
 * \brief 表单模板。常用做表单展示。
 */

TopTemplateClass2::TopTemplateClass2(const QString &iModuleName,
                                     const QVariantMap &iUrlPars,
                                     QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    initUi();
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    QTimer::singleShot(0, this, SLOT(refresh()));
}

TopTemplateClass2::~TopTemplateClass2()
{
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 设置表单，\a iKey 为 该模块中对应的ui.js文件
 */
void TopTemplateClass2::setUiKey(const QString &iKey)
{
    if (mUiLoader == nullptr) {
        return;
    }
    mUiLoader->setUiStr(ui(iKey).toString());
}

/*!
 * \brief 设置表单，\a iUi 为表单界面的字符串
 */
void TopTemplateClass2::setUiStr(const QString &iUi)
{
    if (mUiLoader == nullptr) {
        return;
    }
    mUiLoader->setUiStr(iUi);
}

/*!
 * \brief 刷新数据
 */
void TopTemplateClass2::refresh()
{
    QString sql = config("uiloader.data_set.db_sql").toString();
    if (sql.isEmpty()) {
        QString tableName = config("uiloader.data_set.db_table_name").toString();
        if (!tableName.isEmpty()) {
            TSqlSelectorV2 selector;
            selector.setTable(tableName);
            selector.setField(TopClassHelper::fetchQueryFieldsFromUi(mUiLoader));
            QVariantMap filterMap = config("uiloader.data_set.db_filter").toMap();
            if (!filterMap.isEmpty()) {
                QMapIterator<QString, QVariant> i(filterMap);
                while (i.hasNext()) {
                    i.next();
                    selector.addWhere(i.key(), i.value());
                }
            }
            TError err;
            QVariantMap data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP, QVariant::fromValue(selector), &err).toMap();
            if (err.isValid()) {
                alertError(ttr("Load data failed!"), err.text());
                return;
            }
            mUiLoader->loadValues(data);
        }
    } else {
        TError err;
        QVariantMap data = TConcurrent::runSync([this, &err, sql](){
            TSqlQueryV2 query(T_SQLCNT_POOL->getSqlDatabase());
            QVariantMap v = query.selectMap(sql, QVariantMap());
            if (query.lastError().isValid()) {
                err = query.lastError();
                return QVariantMap();
            }
            return v;
        }).toMap();
        if (err.isValid()) {
            alertError(ttr("Load data failed!"), err.text());
            return;
        }
        mUiLoader->loadValues(data);
    }
}

/*!
 * \brief 表单的查询数据库的字段
 */
QStringList TopTemplateClass2::queryFields() const
{
    return TopClassHelper::fetchQueryFieldsFromUi(mUiLoader);
}

/*!
 * \brief 表单控件
 */
TUiLoader *TopTemplateClass2::uiLoader()
{
    return mUiLoader;
}

/*!
 * \brief 表单当前的数据
 */
QVariant TopTemplateClass2::uiLoaderValues() const
{
    return mUiLoader->getAllValues(true).toVariant();
}

/*!
 * \brief 当表单数据发生改变时的槽函数
 */
void TopTemplateClass2::onDataChanged()
{
    setDataModified(true);
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass2::initUi()
{
    QWidget *bodyWgt = new QWidget(this);
    setCentralWidget(bodyWgt);

    QVBoxLayout *bodyLayout = new QVBoxLayout(bodyWgt);
    bodyLayout->setSpacing(0);
    bodyLayout->setMargin(0);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
    }

    mUiLoader = new TUiLoader(this);
    const QStringList typePlugins = config("uiloader_plugins").toStringList();
    if (typePlugins.size() > 0) {
        TopClassHelper::setUiLoaderPluginCallback(mUiLoader,this,typePlugins);
    }
    connect(mUiLoader, SIGNAL(dataChanged()), this, SLOT(onDataChanged()));
    bodyLayout->addWidget(mUiLoader, 1);

    mUiLoader->setScriptEngine(APP->scriptEngine());
    mUiLoader->setSelf(this);

    QString uiStr = config("uiloader.ui_str").toString();
    if (uiStr.isEmpty()) {
        uiStr = ui(config("uiloader.ui_key").toString()).toString();
    }
    setUiStr(uiStr);
}
