﻿#include "toptemplateclass3.h"
#include <QtWidgets>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tbaseutil/ttheme.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <twidget/tsplitter.h>
#include <twidget/ttableview.h>
#include <twidget/tsearchentry.h>
#include <twidget/tpagetool.h>
#include <twidget/tuiloader.h>

/*!
 * \class TopTemplateClass3
 * \inmodule templateclass
 * \brief 带详细信息的单表展示。支持新建、复制新建、删除。
 */

TopTemplateClass3::TopTemplateClass3(const QString &iModuleName,
                                     const QVariantMap &iUrlPars,
                                     QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
    initUi();
    connect(this, SIGNAL(detailSaved(QString)),
            this, SLOT(onDetailSaved(QString)));
    connect(this, SIGNAL(isDetailModifiedChanged(bool)),
            this, SLOT(onDetailChanged(bool)));
    connect(this, SIGNAL(detailUidChanged(QString)),
            this, SLOT(refreshDetail(QString)));
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mBodySplitter);
    restoreObjectState(mTableView);
    refreshActionState();
    QTimer::singleShot(0, this, SLOT(refresh()));

    auto refreshInterval = config("view.timing_refresh_interval", 0).toInt();
    if (refreshInterval > 0) {
        mRefreshTimer = new QTimer(this);
        connect(mRefreshTimer, SIGNAL(timeout()), this, SLOT(refresh()));
        mRefreshTimer->start(refreshInterval * 1000);
    }
}

TopTemplateClass3::~TopTemplateClass3()
{
    saveSizeState();
    saveObjectState(mBodySplitter);
    saveObjectState(mTableView);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
    if (mRefreshTimer != nullptr) {
        if (mRefreshTimer->isActive()) {
            mRefreshTimer->stop();
        }
        mRefreshTimer->deleteLater();
    }
}

/*!
 * \brief 返回详细信息模块是否发生数据改变
 */
bool TopTemplateClass3::isDetailModified() const
{
    return mIsDetailModified;
}

/*!
 * \brief 刷新数据，\a iResetPage 表示是否要重置分页
 */
void TopTemplateClass3::refresh(bool iResetPage)
{
    TSqlSelectorV2 selector;
    if (!mTableConf.dbSql.isEmpty()) {
        selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTableConf.dbSql));
    } else {
        selector.setTable(mTableConf.dbTableName);
    }
    selector.setField(mTableConf.queryFields);
    selector.setReturnRowCount(true);
    selector.setFieldFormat(mTableConf.fieldFormatMap);
    // 搜索分页
    TopClassHelper::handleSearchPageOnQuery(mSearchEntry, mPageTool, iResetPage, &selector);
    // 排序
    TopClassHelper::handleOrderOnQuery(mTableView, &mTableConf, &selector);
    // 只获取有效数据
    if (!mTableConf.dbDelFlagKey.isEmpty()) {
        selector.addWhere(QString("%1 = 0 OR %1 IS NULL").arg(mTableConf.dbDelFlagKey));
    }
    // 过滤项
    if(!mTableConf.dbFilter.isEmpty()) {
        selector.addWhere(mTableConf.dbFilter);
    }
    loading(ttr("Loading"));
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
    unloading();
    TDataResponse response(data.toMap());
    if (response.hasError()) {
        alertError(ttr("Load data failed!"));
    } else {
        if (mPageTool != nullptr) {
            mPageTool->setRowCount(response.dataCount(), true);
        }
        QVariantList tableData = response.data().toList();
        TopClassHelper::formatTableData(this, &mTableConf, tableData);
        mTableView->loadData(tableData);
        mSelectedItems.clear();
        alertOk(ttr("Data loaded"));
        if (isHookExists("afterDataRefresh")) {
            callHooks("afterDataRefresh");
        }
    }
}

/*!
 * \brief 刷新详细信息
 */
void TopTemplateClass3::refreshDetail(const QString &iUid)
{
    if (iUid.isEmpty()) {
        mDetailView->loadValues(QVariantMap(), true);
    } else {
        if (!mTableConf.dbTableName.isEmpty()) {
            TSqlSelectorV2 selector;
            selector.setTable(mTableConf.dbTableName);
            selector.setField(mTableConf.queryFields);
            selector.setWhere(mTableConf.primaryKey, iUid);
            selector.setFieldFormat(mTableConf.fieldFormatMap);
            TError err;
            QVariant data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP,
                                                    QVariant::fromValue(selector),
                                                    &err);
            if (err.isValid()) {
                alertError(ttr("Load data failed!"), err.text());
                return;
            }
            QVariantMap dataMap = data.toMap();
            mDetailView->loadValues(dataMap, true);
        }
    }
    setIsDetailModified(false);
}

/*!
 * \brief 当前的详细信息的UID
 */
QString TopTemplateClass3::detailUid() const
{
    return mDetailUid;
}

/*!
 * \brief 设置详细信息的UID为 \a iUid,
 * 如果 \a iForceRefresh 为 true, 表示无论UID发生改变，都强制刷新
 */
void TopTemplateClass3::setDetailUid(const QString &iUid, bool iForceRefresh)
{
    if (iForceRefresh || mDetailUid != iUid) {
        mDetailUid = iUid;
        emit detailUidChanged(iUid);
    }
}

/*!
 * \brief 设置详细信息的数据改变状态
 */
void TopTemplateClass3::setIsDetailModified(bool iIsDetailModified)
{
    if (mIsDetailModified == iIsDetailModified) {
        return;
    }
    mIsDetailModified = iIsDetailModified;
    emit isDetailModifiedChanged(iIsDetailModified);
}

/*!
 * \brief 当前选中条目的primaryKey
 */
QVariantList TopTemplateClass3::selectedItems() const
{
    return mSelectedItems;
}

/*!
 * \brief 当前选中条目的所有数据
 */
QVariantList TopTemplateClass3::selectedDataMaps() const
{
    return mTableView->selectedRowDataMaps(true);
}

/*!
 * \brief 新建条目
 */
void TopTemplateClass3::newItem()
{
    mLastDetailUid = mDetailUid;
    setDetailUid(QString());
    setIsDetailModified(true);
}

/*!
 * \brief 保存条目
 */
void TopTemplateClass3::saveItem()
{
    QVariantList errLst = mDetailView->validateAll("COMMIT", true, "ERROR");
    if (!errLst.isEmpty()) {
        QStringList errStrLst;
        for (QVariant err : errLst) {
            errStrLst.append(err.toMap().value("text").toString());
        }
        alertError(ttr("Save data failed!"), errStrLst.join("\n"));
        return;
    }
    TSqlInserterV2 inserter;
    inserter.setTable(mTableConf.dbTableName);
    QVariantMap saveData;
    QMap<QString, QString> updatePolicy;
    TopClassHelper::fetchSaveFieldsFromUi(mDetailView, saveData, updatePolicy);
    inserter.setUniqueField(mTableConf.primaryKey);
    inserter.setAutoIncrementField(mTableConf.primaryKey);
    saveData.insert(mTableConf.primaryKey, mDetailUid);
    // 添加过滤项数据
    if (config("view.data_set.db_filter").type() == QVariant::Map) {
        QVariantMap dbFilterMap = config("view.data_set.db_filter").toMap();
        QMapIterator<QString, QVariant> i(dbFilterMap);
        while (i.hasNext()) {
            i.next();
            saveData.insert(i.key(), i.value());
        }
    }
    inserter.setData(saveData);
    QStringList fields = saveData.keys();
    fields.removeOne(mTableConf.primaryKey);
    inserter.setField(fields);
    // 设置更新策略
    {
        QMapIterator<QString, QString> i(updatePolicy);
        while (i.hasNext()) {
            i.next();
            inserter.setUpdatePolicy(i.key(), i.value());
        }
    }
    TError err;
    QVariant result = runSqlQueryOnThreadSync(TOPSQLTHREAD_REPLACE_ROW,
                                              QVariant::fromValue(inserter),
                                              &err);
    if (err.isValid()) {
        alertError(ttr("Save data failed!"), err.text());
        return;
    }
    setIsDetailModified(false);
    emit detailSaved(result.toString());
    alertOk(ttr("Data saved"));
}

/*!
 * \brief 重新载入条目
 */
void TopTemplateClass3::reloadItem()
{
    setDetailUid(mLastDetailUid.isEmpty() ? mDetailUid : mLastDetailUid, true);
}

/*!
 * \brief 复制条目
 */
void TopTemplateClass3::copyItem()
{
    mLastDetailUid = mDetailUid;
    mDetailUid = "";
    setIsDetailModified(true);
}

/*!
 * \brief 删除条目
 */
void TopTemplateClass3::deleteItems(const QVariantList &iItems)
{
    if (mTableConf.dbDelFlagKey.isEmpty()) {
        // 物理删除
        TSqlDeleterV2 deleter;
        deleter.setTable(mTableConf.dbTableName);
        deleter.setWhere(mTableConf.primaryKey, iItems);
        TError err;
        runSqlQueryOnThreadSync(TOPSQLTHREAD_DELETE_ROW,
                                QVariant::fromValue(deleter),
                                &err);
        if (err.isValid()) {
            alertError(ttr("Delete data failed!"), err.text());
        } else {
            refresh(true);
            alertOk(ttr("Data deleted"));
        }
    } else {
        // 逻辑删除
        TSqlUpdaterV2 updater;
        updater.setTable(mTableConf.dbTableName);
        updater.setWhere(mTableConf.primaryKey, iItems);
        updater.setData(QVariantMap {{mTableConf.dbDelFlagKey, 1}});
        TError err;
        runSqlQueryOnThreadSync(TOPSQLTHREAD_UPDATE_ROW,
                                QVariant::fromValue(updater),
                                &err);
        if (err.isValid()) {
            alertError(ttr("Delete data failed!"), err.text());
        } else {
            refresh(true);
            alertOk(ttr("Moved to recycle bin"));
        }
    }
}

/*!
 * \brief 表格控件对象
 */
TTableView *TopTemplateClass3::tableView() const
{
    return mTableView;
}

/*!
 * \brief 详细信息的表单对象
 */
TUiLoader *TopTemplateClass3::uiLoader() const
{
    return mDetailView;
}

/*!
 * \brief 当选中发生改变时的槽函数
 */
void TopTemplateClass3::onSelectionChanged()
{
    mSelectedItems = mTableView->selectedPrimaryKeys();
    QString curId;
    if (!mSelectedItems.isEmpty()) {
        curId = mSelectedItems.first().toString();
    }
    if (mDetailUid != curId) {
        mLastDetailUid = curId;
        if (!mIsDetailModified) {
            setDetailUid(curId);
        }
    }
    refreshActionState();
}

/*!
 * \brief 当分页发生改变时的槽函数
 */
void TopTemplateClass3::onPageChanged()
{
    refresh(false);
}

/*!
 * \brief 当详细信息表单数据发生改变时的槽函数
 */
void TopTemplateClass3::onDetailDataChanged()
{
    setIsDetailModified(true);
}

/*!
 * \brief 当详细信息表单保存成功时的槽函数
 */
void TopTemplateClass3::onDetailSaved(const QString &iUid)
{
    refresh(false);
    mTableView->selectRow(iUid);
}

/*!
 * \brief 当详细信息表单数据改变状态发生变化时的槽函数
 */
void TopTemplateClass3::onDetailChanged(bool iIsDetailModified)
{
    mTableView->setEnabled(!iIsDetailModified);
    if (iIsDetailModified) {
        showMaskFrame(mBodyWgt);
    } else {
        hideMaskFrame();
    }
    refreshActionState();
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass3::initUi()
{
    mBodySplitter = new TSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);

    mBodyWgt = new QWidget(this);
    QVBoxLayout *bodyLayout = new QVBoxLayout(mBodyWgt);
    bodyLayout->setMargin(0);
    bodyLayout->setSpacing(0);

    mTableView = new TTableView(this);
    mTableView->setObjectName("mTableView");
    connect(mTableView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refresh()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SIGNAL(selectionChanged()));
    connect(this, SIGNAL(selectionChanged()),
            this, SLOT(onSelectionChanged()));
    mTableView->setDataKeyList(mTableConf.dataKeys);
    mTableView->setPrimaryKey(mTableConf.primaryKey);
    QVariantList headerItems = mTableConf.horizontalHeaders;
    headerItems.prepend(QVariant());
    mTableView->setHeaderItem(headerItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
        if (mSearchEntry != nullptr) {
            connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refresh()));
            mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mTableView->headerItem()));
            mSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("TABLEVIEW_POPUP"))) {
        mTableView->setContextMenu(tablePopup);
    }

    bodyLayout->addWidget(mTableView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mPageTool = qobject_cast<TPageTool *>(uim()->getWidget("BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mPageTool != nullptr) {
            mPageTool->setPageSizeVisible(true);
            connect(mPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onPageChanged()));
        }
    }

    mBodySplitter->addWidget(mBodyWgt);

    QWidget *detailWgt = new QWidget(this);
    QVBoxLayout *detailLayout = new QVBoxLayout(detailWgt);
    detailLayout->setSpacing(0);
    detailLayout->setMargin(0);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("DETAIL_TOOLBAR"))) {
        detailLayout->addWidget(toolbar, 0);
    }
    mDetailView = new TUiLoader(this);
    const QStringList typePlugins = config("uiloader_plugins").toStringList();
    if (typePlugins.size() > 0) {
        TopClassHelper::setUiLoaderPluginCallback(mDetailView,this,typePlugins);
    }
    connect(mDetailView, SIGNAL(dataChanged()), this, SLOT(onDetailDataChanged()));
    mDetailView->setProperty("SS_BG", "PANEL");
    mDetailView->setSelf(this);
    mDetailView->setScriptEngine(APP->scriptEngine());
    mDetailView->setUiStr(ui("detail").toString());
    detailLayout->addWidget(mDetailView);

    mBodySplitter->addWidget(detailWgt);

    mBodySplitter->setStretchFactor(0, 1);
    mBodySplitter->setSizes(QList<int>{1, config("detail.perfect_width", 600).toInt()});
}
