﻿#include "toptemplateclass5.h"
#include <QtWidgets>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tbaseutil/ttheme.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <twidget/tsplitter.h>
#include <twidget/ttableview.h>
#include <twidget/tsearchentry.h>
#include <twidget/tpagetool.h>
#include <twidget/tuiloader.h>

#include <QRegularExpression>
namespace {
    QString nameWidthTablePrefix(const QString &iPrefix, const QString &iOldName) {
        QRegularExpression reg{QString(R"(table\(%1\))").arg(iPrefix)};
        QRegularExpressionMatch match = reg.match(iOldName);
        if(match.hasMatch()) {
            const QString matchStr = match.captured();
            return iOldName;
        } else {
            return QString("table(%1).%2").arg(iPrefix).arg(iOldName);
        }
    }

    void getPageToolData(TPageTool *iPageTool, const bool iResetPage, int &pageNum, int &pageSize) {
        if(iPageTool) {
            pageNum = 1;
            pageSize = -1;
            if(iResetPage) {
                iPageTool->setCurrentPage(1,true);
            }
            pageNum = iPageTool->currentPage();
            pageSize = iPageTool->pageSize();
        }
        if(pageNum < 1) { pageNum = 1; }
        if(pageSize < 1) { pageSize = 100; }
    }
}

/*!
 * \class TopTemplateClass5
 * \inmodule templateclass
 * \brief 主从表模板，左右结构，用两个TableView展示。
 */

TopTemplateClass5::TopTemplateClass5(const QString &iModuleName,
                                     const QVariantMap &iUrlPars,
                                     QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "master_view", &mMasterTableConf);
    TopClassHelper::parseTableConf0(this, "slave_view", &mSlaveTableConf);
    initUi();
    connect(this, SIGNAL(isDetailModifiedChanged(bool)),
            this, SLOT(onDetailChanged(bool)));
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mLeftSplitter);
    restoreObjectState(mBodySplitter);
    restoreObjectState(mMasterView);
    restoreObjectState(mSlaveView);
    refreshActionState();
    QTimer::singleShot(0, this, SLOT(refreshMaster()));
}

TopTemplateClass5::~TopTemplateClass5()
{
    saveSizeState();
    saveObjectState(mLeftSplitter);
    saveObjectState(mBodySplitter);
    saveObjectState(mMasterView);
    saveObjectState(mSlaveView);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 刷新主表的数据，\a iResetPage 为true时表示重置分页
 */
void TopTemplateClass5::refreshMaster(bool iResetPage)
{
    QString sql = config("master_view.data_set.db_sql").toString();
    if (sql.isEmpty()) {
        if (!mMasterTableConf.dbTableName.isEmpty()) {
            TSqlSelectorV2 selector;
            selector.setTable(mMasterTableConf.dbTableName);
            selector.setField(mMasterTableConf.queryFields);
            selector.setReturnRowCount(true);
            // 搜索分页
            TopClassHelper::handleSearchPageOnQuery(mMasterSearchEntry, mMasterPageTool, iResetPage, &selector);
            // 排序
            TopClassHelper::handleOrderOnQuery(mMasterView, &mMasterTableConf, &selector);
            loading(ttr("Loading"));
            QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
            unloading();
            TDataResponse response(data.toMap());
            if (response.hasError()) {
                alertError(ttr("Load data failed!"));
            } else {
                if (mMasterPageTool != nullptr) {
                    mMasterPageTool->setRowCount(response.dataCount(), true);
                }
                QVariantList tableData = response.data().toList();
                TopClassHelper::formatTableData(this, &mMasterTableConf, tableData);
                mMasterView->loadData(tableData);
                alertOk(ttr("Data loaded"));
            }
        } else {
            // sql语句为空，dbname也为空时执行钩子获取master数据
            if (isHookExists("masterWillUpdate")) {
                int pageNum = 1;
                int pageSize = -1;
                getPageToolData(mMasterPageTool,iResetPage,pageNum,pageSize);
                QScriptValue argVal = APP->scriptEngine()->newObject();
                argVal.setProperty("resetPage", iResetPage);
                argVal.setProperty("pageNum", pageNum);
                argVal.setProperty("pageSize", pageSize);
                callHooks("masterWillUpdate", QScriptValueList() << argVal);
            }
            TopClassHelper::formatTableData(this, &mMasterTableConf, mMasterData);
            mMasterView->loadData(mMasterData);
            alertOk(ttr("Data loaded"));
        }
    } else {
        // @todo
    }
}

/*!
 * \brief 刷新从表的数据，\a iResetPage 为true时表示重置分页
 */
void TopTemplateClass5::refreshSlave(bool iResetPage)
{
    if(selectedMasterItems().size() == 0) {
        mSlaveView->loadData(QVariantList());
        if (mUiLoader) {
            mUiLoader->loadValues(QVariant());
        }
        return;
    }

    QString sql = config("slave_view.data_set.db_sql").toString();
    if (sql.isEmpty()) {
        if (!mSlaveTableConf.dbTableName.isEmpty()) {
            QString relationTable = config("master_view.relation.through").toString();
            QString slaveKey = config("master_view.relation.slave_key").toString();
            QString slaveForeignKey = config("master_view.relation.slave_foreign_key").toString();
            QString masterForeignKey = config("master_view.relation.master_foreign_key").toString();
            QStringList fields = mSlaveTableConf.queryFields;
            for (auto & f : fields) {
                f = QString("S.%1 AS %1").arg(f);
            }
            TSqlSelectorV2 selector;
            QString table = QString("%1 AS S LEFT JOIN %2 AS R ON S.%3 = R.%4")
                    .arg(mSlaveTableConf.dbTableName)
                    .arg(relationTable)
                    .arg(slaveKey)
                    .arg(slaveForeignKey);
            selector.setTable(table);
            selector.setField(fields);
            selector.setWhere("R."+masterForeignKey, mSelectedMasterItems.first());
            // 搜索分页
            TopClassHelper::handleSearchPageOnQuery(mSlaveSearchEntry, mSlavePageTool, iResetPage, &selector);
            // 排序
            TopClassHelper::handleOrderOnQuery(mSlaveView, &mSlaveTableConf, &selector);
            loading(ttr("Loading"));
            QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
            unloading();
            TDataResponse response(data.toMap());
            if (response.hasError()) {
                alertError(ttr("Load data failed!"));
            } else {
                if (mSlavePageTool != nullptr) {
                    mSlavePageTool->setRowCount(response.dataCount(), true);
                }
                QVariantList tableData = response.data().toList();
                TopClassHelper::formatTableData(this, &mSlaveTableConf, tableData);
                mSlaveView->loadData(tableData);
                alertOk(ttr("Data loaded"));
            }
        } else {
            // sql语句为空，dbname也为空时执行钩子获取slave数据
            QString selectedPrimaryKey = mSelectedMasterItems.first().toString();
            if (isHookExists("slaveWillUpdate")) {
                callHooksQuick("slaveWillUpdate", QVariantList() << selectedPrimaryKey);
            }
            QVariantList tableData = mMasterSlaveDataMap.value(selectedPrimaryKey).toList();
            TopClassHelper::formatTableData(this, &mSlaveTableConf, tableData);
            mSlaveView->loadData(tableData);
            alertOk(ttr("Data loaded"));
        }
    } else {
        // @todo
    }

    refreshDetail();
}

void TopTemplateClass5::refreshDetail()
{
    if (mUiLoader != nullptr) {
        QVariantMap dataMap;
        const QVariantMap masterMap = selectedMasterDataMaps().value(0).toMap();
        for(const QString &key: masterMap.keys()) {
            const QString tKey = ::nameWidthTablePrefix("master",key);
            dataMap.insert(tKey,masterMap[key]);
        }

        QVariantMap slaveMap = selectedSlaveDataMaps().value(0).toMap();
        for(const QString &key: slaveMap.keys()) {
            const QString tKey = ::nameWidthTablePrefix("slave",key);
            dataMap.insert(tKey,slaveMap[key]);
        }

        mUiLoader->loadValues(QVariant(dataMap));
        mUiLoader->refreshState();
    }
    setIsDetailModified(false);
}

/*!
 * \brief 主表格控件对象
 */
TTableView *TopTemplateClass5::masterView() const
{
    return mMasterView;
}

/*!
 * \brief 从表格控件对象
 */
TTableView *TopTemplateClass5::slaveView() const
{
    return mSlaveView;
}

const QStringList TopTemplateClass5::masterQueryFields() const
{
    return mMasterTableConf.queryFields;
}

const QStringList TopTemplateClass5::slaveQueryFields() const
{
    return mSlaveTableConf.queryFields;
}

/*!
 * \brief 主表当前选中条目的primaryKey
 */
QVariantList TopTemplateClass5::selectedMasterItems() const
{
    return mSelectedMasterItems;
}

/*!
 * \brief 主表当前选中条目的所有数据
 */
QVariantList TopTemplateClass5::selectedMasterDataMaps() const
{
    return mMasterView->selectedRowDataMaps(true);
}

/*!
 * \brief 从表当前选中条目的primaryKey
 */
QVariantList TopTemplateClass5::selectedSlaveItems() const
{
    return mSelectedSlaveItems;
}

/*!
 * \brief 从表当前选中条目的所有数据
 */
QVariantList TopTemplateClass5::selectedSlaveDataMaps() const
{
    return mSlaveView->selectedRowDataMaps(true);
}

/*!
 * \brief 导入树形结构的数据
 * {
 *   "A": {
 *     "master_title": "",
 *     "data": [
 *       {
 *         "slave_id": "",
 *         "slave_name": ""
 *       }
 *     ]
 *   }
 * }
 */
void TopTemplateClass5::loadTreeData(const QVariantMap &iTreeData,
                                     const QString &iSlavesKey)
{
    mMasterData.clear();
    mMasterSlaveDataMap.clear();
    QMapIterator<QString, QVariant> i(iTreeData);
    while (i.hasNext()) {
        i.next();
        QVariantMap item = i.value().toMap();
        mMasterSlaveDataMap.insert(i.key(), item.value(iSlavesKey));
        item.remove(iSlavesKey);
        item.insert("name", i.key());
        mMasterData.append(item);
    }
    refreshMaster();
}

/*!
 * \brief 导入表格结构的数据
 * \todo 待实现
 */
void TopTemplateClass5::loadTableData(const QVariantList &iTableData,
                                      const QString &iSlavesKey)
{
    Q_UNUSED(iTableData)
    Q_UNUSED(iSlavesKey)
    // @todo
}

void TopTemplateClass5::loadMasterData(const QVariantList &iMasterData, int iRowCount)
{
    mMasterData = iMasterData;
    if(mMasterPageTool) {
        mMasterPageTool->setRowCount(iRowCount,true);
    }
}

void TopTemplateClass5::insertSlaveData(const QString &iKey, const QVariant &iValue)
{
    mMasterSlaveDataMap.insert(iKey, iValue);
}

TUiLoader *TopTemplateClass5::uiLoader() const
{
    return mUiLoader;
}

void TopTemplateClass5::selectSlaveRow(const QVariant &iKeyVar)
{
    mSlaveView->selectRow(iKeyVar);
}

/*!
 * \brief 返回详细信息模块是否发生数据改变
 */
bool TopTemplateClass5::isDetailModified() const
{
    return mIsDetailModified;
}

/*!
 * \brief 设置详细信息的数据改变状态
 */
void TopTemplateClass5::setIsDetailModified(bool iIsDetailModified)
{
    if (mIsDetailModified == iIsDetailModified) {
        return;
    }
    mIsDetailModified = iIsDetailModified;
    emit isDetailModifiedChanged(iIsDetailModified);
}

/*!
 * \brief 当主表选中发生改变的槽函数
 */
void TopTemplateClass5::onMasterSelectionChanged()
{
    mSelectedMasterItems = mMasterView->selectedPrimaryKeys();
    refreshSlave();
    refreshActionState();
}

/*!
 * \brief 当主表分页发生改变的槽函数
 */
void TopTemplateClass5::onMasterPageChanged()
{
    refreshMaster(false);
}

/*!
 * \brief 当从表选中发生改变的槽函数
 */
void TopTemplateClass5::onSlaveSelectionChanged()
{
    mSelectedSlaveItems = mSlaveView->selectedPrimaryKeys();
    refreshDetail();
    refreshActionState();
}

/*!
 * \brief 当从表分页发生改变的槽函数
 */
void TopTemplateClass5::onSlavePageChanged()
{
    refreshSlave(false);
}

void TopTemplateClass5::onDetailChanged(bool iIsDetailModified)
{
    if (iIsDetailModified) {
        showMaskFrame(mLeftSplitter);
    } else {
        hideMaskFrame();
    }
    refreshActionState();
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass5::initUi()
{
    mBodySplitter = new TSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);

    mLeftSplitter = new TSplitter(this);
    mLeftSplitter->setObjectName("mLeftSplitter");
    mBodySplitter->addWidget(mLeftSplitter);

    // 主表
    QFrame *masterWgt = new QFrame(this);
    mLeftSplitter->addWidget(masterWgt);

    QVBoxLayout *masterLayout = new QVBoxLayout(masterWgt);
    masterLayout->setMargin(0);
    masterLayout->setSpacing(0);

    mMasterView = new TTableView(this);
    mMasterView->setObjectName("mMasterView");
    connect(mMasterView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refreshMaster()));
    connect(mMasterView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onMasterSelectionChanged()));
    mMasterView->setDataKeyList(mMasterTableConf.dataKeys);
    mMasterView->setPrimaryKey(mMasterTableConf.primaryKey);
    QVariantList masterHeaderItems = mMasterTableConf.horizontalHeaders;
    masterHeaderItems.prepend(QVariant());
    mMasterView->setHeaderItem(masterHeaderItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MASTER_TOOLBAR"))) {
        masterLayout->addWidget(toolbar, 0);
        mMasterSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MASTER_TOOLBAR/SEARCH_ENTRY"));
        if (mMasterSearchEntry != nullptr) {
            connect(mMasterSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refreshMaster()));
            mMasterSearchEntry->setOptionList(TDataParse::headerItem2searchList(mMasterView->headerItem()));
            mMasterSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("MASTER_POPUP"))) {
        mMasterView->setContextMenu(tablePopup);
    }

    masterLayout->addWidget(mMasterView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MASTER_BOTTOM_TOOLBAR"))) {
        masterLayout->addWidget(toolbar, 0);
        mMasterPageTool = qobject_cast<TPageTool *>(uim()->getWidget("MASTER_BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mMasterPageTool != nullptr) {
            mMasterPageTool->setPageSizeVisible(true);
            connect(mMasterPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onMasterPageChanged()));
        }
    }

    // 从表
    QWidget *slaveWgt = new QWidget(this);
    mLeftSplitter->addWidget(slaveWgt);

    QVBoxLayout *slaveLayout = new QVBoxLayout(slaveWgt);
    slaveLayout->setMargin(0);
    slaveLayout->setSpacing(0);

    mSlaveView = new TTableView(this);
    mSlaveView->setObjectName("mSlaveView");
    connect(mSlaveView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refreshSlave()));
    connect(mSlaveView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSlaveSelectionChanged()));
    mSlaveView->setDataKeyList(mSlaveTableConf.dataKeys);
    mSlaveView->setPrimaryKey(mSlaveTableConf.primaryKey);
    QVariantList slaveHeaderItems = mSlaveTableConf.horizontalHeaders;
    slaveHeaderItems.prepend(QVariant());
    mSlaveView->setHeaderItem(slaveHeaderItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("SLAVE_TOOLBAR"))) {
        slaveLayout->addWidget(toolbar, 0);
        mSlaveSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("SLAVE_TOOLBAR/SEARCH_ENTRY"));
        if (mSlaveSearchEntry != nullptr) {
            connect(mSlaveSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refreshSlave()));
            mSlaveSearchEntry->setOptionList(TDataParse::headerItem2searchList(mSlaveView->headerItem()));
            mSlaveSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("SLAVE_POPUP"))) {
        mSlaveView->setContextMenu(tablePopup);
    }

    slaveLayout->addWidget(mSlaveView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("SLAVE_BOTTOM_TOOLBAR"))) {
        slaveLayout->addWidget(toolbar, 0);
        mSlavePageTool = qobject_cast<TPageTool *>(uim()->getWidget("SLAVE_BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mSlavePageTool != nullptr) {
            mSlavePageTool->setPageSizeVisible(true);
            connect(mSlavePageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onSlavePageChanged()));
        }
    }

    // 详情界面
    const QString uiStr = ui("detail").toString();
    if(!uiStr.isEmpty()) {
        QWidget *detailWgt = new QWidget(this);
        mBodySplitter->addWidget(detailWgt);
        QVBoxLayout *detailVLayout = new QVBoxLayout(detailWgt);
        detailVLayout->setMargin(0);
        detailVLayout->setSpacing(0);

        if(QToolBar *toolBar = qobject_cast<QToolBar*>(uim()->getWidget("DETAIL_TOOLBAR"))) {
            detailVLayout->addWidget(toolBar);
        }

        mUiLoader = new TUiLoader(detailWgt);
        const QStringList typePlugins = config("uiloader_plugins").toStringList();
        if (typePlugins.size() > 0) {
            TopClassHelper::setUiLoaderPluginCallback(mUiLoader,this,typePlugins);
        }
        mUiLoader->setProperty("SS_BG","PANEL");
        mUiLoader->setSelf(this);
        mUiLoader->setScriptEngine(APP->scriptEngine());
        mUiLoader->setUiStr(uiStr);
        connect(mUiLoader, &TUiLoader::dataChanged, [this](){
            setIsDetailModified(true);
        });
        detailVLayout->addWidget(mUiLoader);
    }

    resizeSplitterByRatio();
}

void TopTemplateClass5::resizeSplitterByRatio()
{
    QString masterSlaveRatio = config("master_slave_ratio", "1:1:1").toString();
    QRegularExpression regExp("(?<master>\\d+):(?<slave>\\d+):(?<uiloader>\\d+)");
    QRegularExpressionMatch match = regExp.match(masterSlaveRatio);
    if (match.hasMatch()) {
        double m = match.captured("master").toDouble();
        double s = match.captured("slave").toDouble();
        double u = match.captured("uiloader").toDouble();
        if (mUiLoader == nullptr) {
            u = 0.0;
        }
        double leftRatio =  (m + s) / (m + s + u);
        int width = this->width();
        int leftWidth =  static_cast<int>(width * leftRatio);
        mBodySplitter->setSizes(QList<int>{ leftWidth, width-leftWidth });

        double leftRatio1 = m / (m + s);
        int leftWidth1 = leftWidth * leftRatio1;
        mLeftSplitter->setSizes(QList<int>{ leftWidth1, leftWidth - leftWidth1 });
    }
}
